package com.persagy.gateway.filter;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.util.Set;

import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;

import com.alibaba.fastjson.JSONObject;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.StringUtil;
import com.persagy.common.utils.WebfluxResponseUtil;
import com.persagy.gateway.utils.ReactiveAddrUtil;
import com.persagy.log.constant.LogContsant;
import com.persagy.security.constant.CipherConstants;
import com.persagy.security.util.BouncycastleCipher;
import com.persagy.security.util.SecureAES;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.IdUtil;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

/**
 * @version
 * @description 
 * @company persagy
 * @author zhangqiankun
 * @since 2020年10月13日: 下午3:33:52
 */
@Slf4j
@Component
public class CustomRequestPredicateGlobalFilter implements GlobalFilter, Ordered {
	
	private static final String TEST_TOKEN = "9be2851574aee0e310b093bff0e115188a3d020255fa57866cd0b8ca2eca140b.DS865SDFGVX5GN8GE8RDV2B4XB5D8B5";
	
	@Value("${saas.secure.aes.key}")
	private String aesKey;
	
	@Value("${saas.secure.aes.iv}")
	private String aesIv;
	
	@Autowired
	private BouncycastleCipher cipher;
	
	@Override
	public int getOrder() {
		return Ordered.HIGHEST_PRECEDENCE + 1;
	}

	@Override
	public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
		long start = System.currentTimeMillis();

		ServerHttpRequest request = exchange.getRequest();
		ServerHttpResponse response = exchange.getResponse();
		HttpMethod httpMethod = request.getMethod();
		URI uri = request.getURI();
		String remoteAddr = ReactiveAddrUtil.getRemoteAddr(request);
		// 接口预检，放行
		if (HttpMethod.OPTIONS.equals(httpMethod)) {
			return chain.filter(exchange);
		}
		// 非 POST 方式拒绝
		if (!HttpMethod.POST.equals(httpMethod)) {
			return WebfluxResponseUtil.responseFailed(response, ResponseCode.A0135.getCode(), ResponseCode.A0135.getDesc());
		}
		HttpHeaders httpHeaders = request.getHeaders();
		// 根据用户token鉴权
		Mono<Void> result = this.customAuthenticate(response, httpHeaders, uri.getPath());
		if (result != null) {
			return result;
		}
		// 生成链路追踪id
		String traceId = IdUtil.fastSimpleUUID();
		MDC.put(LogContsant.LOG_TRACE_ID, traceId);
		
		return chain.filter(exchange).doFinally((f) -> {
			long end = System.currentTimeMillis();
			log.info("访问IP[{}], 访问链接[{}], 服务总耗时: [{}ms]", remoteAddr, uri.getPath(), (end - start));
			MDC.clear();
		});

	}

	/**
	 * 根据token进行权限认证
	 * 
	 * @param httpHeaders
	 * @param uri 
	 * @return 
	 * @throws UnsupportedEncodingException 
	 */
	private Mono<Void> customAuthenticate(ServerHttpResponse response, HttpHeaders httpHeaders, String uri) {
		// 判断是否需要验证token
		Boolean enable = cipher.getProperties().getTokenEnable();
		if (BooleanUtil.isFalse(enable)) {
			return null;
		}
		// 白名单直接返回true
		Set<String> ignoreUrl = cipher.getProperties().getIgnoreUrl();
		if (CollectionUtil.isNotEmpty(ignoreUrl) && StringUtil.isNotBlank(uri)) {
			for (String ignore : ignoreUrl) {
				if (uri.indexOf(ignore) >= 0) {
					return null;
				}
			}
		}
		// 获取请求token
		String auth = httpHeaders.getFirst(CipherConstants.TOKEN_HEADER);
		if (StringUtil.isBlank(auth)) {
			return WebfluxResponseUtil.responseFailed(response, ResponseCode.A0301.getCode(), ResponseCode.A0301.getDesc());
		}
		// 判断是否为自测，自测的token 不需要进行验证
		if (TEST_TOKEN.equals(auth)) {
			return null;
		}
		// 开始进行token验证
		String[] tokens = auth.split("\\.");
		if (tokens.length < 2) {
			return WebfluxResponseUtil.responseFailed(response, ResponseCode.A0301.getCode(), ResponseCode.A0301.getDesc());
		}
		try {
			// 验证token
			String userInfo = tokens[0];	// 加密的用户信息
			String token = tokens[1];		// token值
			if (StringUtil.isBlank(userInfo) || StringUtil.isBlank(token)) {
				return WebfluxResponseUtil.responseFailed(response, ResponseCode.A0301.getCode(), ResponseCode.A0301.getDesc());
			}
			SecureAES aes = new SecureAES(aesKey, aesIv);
			JSONObject accountInfo = aes.decryptAccount(userInfo);
			String accountId = accountInfo.getString(CipherConstants.ACCOUNT_ID);
			boolean isEqual = cipher.isEqual(token, accountId);
			if (isEqual) {
				return null;
			}
			log.error("request token error ... , token: {}", auth);
		} catch (Exception e) {
			log.error("request token parse error ... , token: {}, message: {}", auth, e.getMessage());
		}
		return WebfluxResponseUtil.responseFailed(response, ResponseCode.A0301.getCode(), ResponseCode.A0301.getDesc());
	}
	
}
